#ifndef _BATTLEMAIN_CPP_
#define _BATTLEMAIN_CPP_

//by Chad Serrant
#include <stdio.h>
#include <math.h>

#include "battleMain.h"

#include "allegro.h"
#include "dataObj.h"
#include "unit.h"
#include "playerUnit.h"
#include "unitList.h"
#include "arena.h"
#include "tasmData.h"
#include "gameConstants.h"
#include "loader.h"
#include <vector>
using namespace std;

////////////////////////
//   INITIALIZATION   //
////////////////////////

volatile int frameTick = 0;
volatile int frameCount = 0;
volatile int frameInRound = 0;
volatile bool gamePaused = false;
volatile bool shouldPauseTimer = false;
bool gameQuit = false;

battleMain :: battleMain ()
{
  dO = NULL;
  buffer = NULL;
  gameOutput = NULL;
  youWin = NULL;
  youLose = NULL;
  introPic = NULL;
  heartPic = NULL;
  clockPic = NULL;

  invicPic = NULL;
  stunnedPic = NULL;
  speedUpPic = NULL;
  speedDownPic = NULL;
  kbDownPic = NULL;
  nojuicePic = NULL;
  morejuicePic = NULL;
  dmgDownPic = NULL;

  picData = NULL;

  battleMode = battleModeIntro;
  displayTimer = 15;
  stageTime = 0;

  //make sure these are the same initially
  battleRound = 0;
  nextRound = 0;

  lder = new loader ();
}

/* timer interrupt handler */
void incFrameTick()
{
  frameTick++;
  if (!shouldPauseTimer)
    frameInRound ++;
}
END_OF_FUNCTION(incFrameTick);

void fpsCalc(void)
{

}
END_OF_FUNCTION(fpsCalc);

int battleMain :: init ()
{
  picData = dO -> setDatafile ("picData.dat");
  srand (time (NULL));

  /*
  FILE *input = fopen( "error", "w" );
  fprintf (input, "start\n");
  fflush (input);
  fclose (input);
  */

  if (picData == NULL)
    {
      printf ("Graphics datafile not found. Now exiting.\n");
      return -1; //data didn't load~!
    }

  //should have a stage loader initialize stage stuff

  set_palette(desktop_palette);
  buffer = create_bitmap(SCREEN_W, SCREEN_H);

  //timer
  LOCK_VARIABLE(frameTick);
  LOCK_VARIABLE(frameCount);
  LOCK_VARIABLE(frameInRound);
  LOCK_VARIABLE(shouldPauseTimer);
  LOCK_FUNCTION (fpsCalc);
  LOCK_FUNCTION(incFrameTick);
  install_int_ex(incFrameTick, BPS_TO_TIMER(30));
  install_int_ex(fpsCalc, BPS_TO_TIMER(1));
  
  youWin = (BITMAP *)picData[youwin].dat;
  youLose = (BITMAP *)picData[youlose].dat;

  heartPic = (BITMAP *)picData[heart].dat;
  clockPic = (BITMAP *)picData[clock].dat;

  invicPic = (BITMAP *)picData[invicIcon].dat;
  stunnedPic = (BITMAP *)picData[stunIcon].dat;
  speedUpPic = (BITMAP *)picData[speedUpIcon].dat;
  speedDownPic = (BITMAP *)picData[speedDownIcon].dat;
  kbDownPic = (BITMAP *)picData[knockbackDownIcon].dat;
  nojuicePic = (BITMAP *)picData[nojuiceIcon].dat;
  morejuicePic = (BITMAP *)picData[juiceUp].dat;
  dmgDownPic = (BITMAP *)picData[damageDownIcon].dat;

  //load graphics and hitbox for units.
  units = new unitList;

  units -> setLoader (lder);
  lder -> init (units, picData);

  restart (true, true);
  dO -> getVideoSettings () -> setObjectPositions (dO -> getArena () -> getArenaSizeX (), dO -> getArena () -> getArenaSizeY ());
  
  //end of temp loading
  
  dO -> getInputObj () -> getCursor () -> setCursorPic ((BITMAP *)picData[targetCursor].dat);

  return 0;
}

void battleMain :: restart (bool gotoNextRound, bool withIntro)
{
  frameCount = 0;
  frameInRound = 0;
  gamePaused = false;
  shouldPauseTimer = false;

  lder -> loadRound (battleRound, nextRound, units, introPic, dO, picData, gotoNextRound);

  gameOutput = create_bitmap (dO -> getArena () -> getArenaSizeX (), dO -> getArena () -> getArenaSizeY ());

  stageTime = units -> latestTime ();
  
  if (!withIntro)
    {
      battleMode = battleModeInProgress;
      displayTimer = 15;
    }
  else
    {
      battleMode = battleModeIntro;
      displayTimer = 15;
    }
  return;
}

  ///////////////////
 /// ACCESSORS   ///
///////////////////
BITMAP * battleMain :: getBuffer ()
{
  return buffer;
}

////////////////////
/// MODIFIERS   ///
///////////////////

void battleMain :: setDataObj (dataObj * dO2)
{
  dO = dO2;
}

//////////////////////////////////////////

void battleMain :: go ()
{
  int resX, resY;
  bool readyToClear = false;

  dO -> getVideoSettings () -> getResolution (resX, resY);

  //check status of the battle.
  if (battleMode == battleModeIntro)
    {
      gamePaused = true;
      displayTimer --;
      shouldPauseTimer = true;

      if (displayTimer < 0)
	{displayTimer = 30; shouldPauseTimer = false; battleMode = battleModeInProgress; gamePaused = false;}
    }
  if (battleMode == battleModeInProgress && frameCount > stageTime)
    battleMode = battleModeTimeUp;

  if ((battleMode == battleModeInProgress || battleMode == battleModeTimeUp) && frameCount > 30 && !units -> anyRemainingAffiliation (affiliationPlayer))
    {
      battleMode = battleModeYouLose;
      displayTimer = 30;
      shouldPauseTimer = true;
    }  
  else if ((battleMode == battleModeInProgress || battleMode == battleModeTimeUp) && frameCount > 30 && !units -> anyRemainingAffiliation (affiliationEnemy))
    {
      battleMode = battleModeYouWin;
      displayTimer = 150;
      shouldPauseTimer = true;
    }

  if (battleMode == battleModeYouWin || battleMode == battleModeYouLose)
    {
      if (!gamePaused)
	displayTimer --;

      //restart the battle, or something.
      if (displayTimer < 0)
	{
	  //if victorious, start next round!
	  if (battleMode == battleModeYouWin)
	    {
	      battleRound = nextRound;
	      restart (true, true);
	    }
	  else if (battleMode == battleModeYouLose)
	    restart (false, false);
	}
    }

  //draw the game
  if (frameTick > 0)
    draw (buffer, gameOutput);

  //cull dead units here, too.
  units -> cullUnits ();

  //translate raw input
  if (frameTick > 0)
    readyToClear = true;

  dO -> getInputObj () -> processInput (resX, resY, readyToClear);

  //apply input
  if (frameTick > 0 && !gamePaused)
    {
      //controls framerate
      applyInput ();
     frameCount++;
    }

  //cull dead units here, too.
  units -> cullUnits ();

  if (dO -> getInputObj () -> getKeyWasClicked () -> at (keyPause))
    {
      gamePaused = !gamePaused;

      if (gamePaused)
	shouldPauseTimer = true;
      else
	shouldPauseTimer = false;
    }

  //check for a restart
  if (dO -> getInputObj () -> getKeyWasClicked () -> at (keyMenu))
    restart (false, false);

  frameTick = 0;
}

  ////////////////
 //  RENDERING //
////////////////

void battleMain :: draw (BITMAP * buffer, BITMAP * gameOutput)
{
  //draw the background
  drawBackground (buffer, gameOutput);

  //draw the units
  drawUnits (gameOutput);

  //place gameOutput on Buffer
  stretch_blit (gameOutput, buffer, 
		0, 0, dO -> getArena () -> getArenaSizeX (), dO -> getArena () -> getArenaSizeY (),
		dO -> getVideoSettings () -> getArenaPosX (), dO -> getVideoSettings () -> getArenaPosY (),
		fixtoi (dO -> getArena () -> getArenaSizeX () * dO -> getVideoSettings () -> getArenaScale ()), 
		fixtoi (dO -> getArena () -> getArenaSizeY () * dO -> getVideoSettings () -> getArenaScale ()));
  
  //draw the cursor...if the game is paused
  if (true)//gamePaused)
    drawCursor (buffer);

  //draw the display
  drawDisplay (buffer);
}

void battleMain :: drawBackground (BITMAP * buffer, BITMAP * gameOutput)
{
  int bk1, bk2, bk3, background;
  
  dO -> getVideoSettings () -> getBackgroundColor (bk1, bk2, bk3);
  background = makecol (bk1, bk2, bk3);
  clear_to_color (buffer, background);

  clear_to_color (gameOutput, makecol (64,64,64));
  //put arena pic in
  if (dO -> getArena () -> getArenaPic () != NULL)
    stretch_sprite (gameOutput, dO -> getArena () -> getArenaPic (), 0, 0, dO -> getArena () -> getArenaSizeX (), dO -> getArena () -> getArenaSizeY ());
}

void battleMain :: drawCursor (BITMAP * buffer)
{
  cursor * cur = dO -> getInputObj () -> getCursor ();
  int curX = 0, curY = 0, curSX = 0, curSY = 0;

  vector <unit *> * mainChar = units -> search (units -> getUnits (), searchByAffiliation, affiliationPlayer);

  cur -> getCursor (curX, curY);
  cur -> getCursorShouldBe (curSX, curSY);

  //change these rules to make new aiming system!
  if (!gamePaused && mainChar -> size () > 0)
    {
      curX = curSX;
      curY = curSY;
    }
  else
    {
      curX = curSX;
      curY = curSY;
    }

  cur -> setCursor (curX, curY);
  //also has to move with unit movement...

  BITMAP * curPic = cur -> getCursorPic ();

  draw_sprite (buffer, curPic, curX - (curPic -> w / 2), curY - (curPic -> h / 2));
}

void battleMain :: drawDisplay (BITMAP * buffer)
{
  float ratiohp = 0.0;

  vector <unit *> * mainChar = units -> search (units -> getUnits (), searchByAffiliation, affiliationPlayer);

  if (mainChar -> size () > 0)
    {
      if (mainChar -> at (0) -> getMaxHP () != 0)
	ratiohp = 1.0 * mainChar -> at (0) -> getHP () / mainChar -> at (0) -> getMaxHP ();

      if (ratiohp < 0)
	ratiohp = 0.0;

      if (ratiohp > 1.0)
	ratiohp = 1.0;
    }

  //drawing hp bar
  drawHPBar (buffer, ratiohp);

  drawTimerBar (buffer, frameInRound);

  //draw statuses
  if (mainChar -> size () > 0)
    drawStatus (buffer, static_cast <playerUnit *> (mainChar -> at (0)));

  //temp instructions
  int tempDisplayY = (dO -> getVideoSettings () -> getResolutionY () / 20 * 17) + 30;

  textprintf_ex(buffer, font, 0, tempDisplayY, makecol (128, 128, 128), -1, "tab pause");
  textprintf_ex(buffer, font, 0, tempDisplayY + 12, makecol (128, 128, 128), -1, "F10 reset");
  textprintf_ex(buffer, font, 0, tempDisplayY + 24, makecol (128, 128, 128), -1, "ESC exit");

  if (battleMode == battleModeIntro)
    {
      //show name of stage
      draw_sprite (buffer, introPic, (dO-> getVideoSettings () -> getResolutionX () - introPic -> w) /2, (dO-> getVideoSettings () -> getResolutionY () - introPic -> h) /2);
    }
  else if (battleMode == battleModeYouLose)
    {
      //no players exist -- you lose!
      draw_sprite (buffer, youLose, (dO-> getVideoSettings () -> getResolutionX () - youWin -> w) /2, (dO-> getVideoSettings () -> getResolutionY () - youWin -> h) /2);
    }  
  else if (battleMode == battleModeYouWin)
    {
      draw_sprite (buffer, youWin, (dO-> getVideoSettings () -> getResolutionX () - youWin -> w) /2, (dO-> getVideoSettings () -> getResolutionY () - youWin -> h) /2);
    }
}

void battleMain :: drawHPBar (BITMAP * buffer, float ratiohp)
{
  int barCol = 0;
  playerUnit * pu = NULL;

  vector <unit *> * mainChar = units -> search (units -> getUnits (), searchByAffiliation, affiliationPlayer);

  if (mainChar -> size () > 0)
    {
      pu = static_cast <playerUnit *> (mainChar -> at (0));
      barCol = pu -> getHPBarColor ();
    }

  rectfill (buffer, dO -> getVideoSettings () -> getHPBarStartX (), dO -> getVideoSettings () -> getHPBarStartY (), 
	dO -> getVideoSettings () -> getHPBarStartX () + dO -> getVideoSettings () -> getHPBarSizeX (), 
	dO -> getVideoSettings () -> getHPBarStartY () + dO -> getVideoSettings () -> getHPBarSizeY (), makecol (0, 0, 0));

  rectfill (buffer, dO -> getVideoSettings () -> getHPBarStartX (), dO -> getVideoSettings () -> getHPBarStartY (), 
	dO -> getVideoSettings () -> getHPBarStartX () + int (dO -> getVideoSettings () -> getHPBarSizeX () * ratiohp), 
	dO -> getVideoSettings () -> getHPBarStartY () + dO -> getVideoSettings () -> getHPBarSizeY (), barCol);

  rect (buffer, dO -> getVideoSettings () -> getHPBarStartX (), dO -> getVideoSettings () -> getHPBarStartY (), 
	dO -> getVideoSettings () -> getHPBarStartX () + dO -> getVideoSettings () -> getHPBarSizeX (), 
	dO -> getVideoSettings () -> getHPBarStartY () + dO -> getVideoSettings () -> getHPBarSizeY (), makecol (128, 128, 128));

  //place pic of heart next to it
  stretch_sprite (buffer, heartPic, dO -> getVideoSettings () -> getHPBarStartX () - dO -> getVideoSettings () -> getHPBarSizeY (), dO -> getVideoSettings () -> getHPBarStartY (), dO -> getVideoSettings () -> getHPBarSizeY (), dO -> getVideoSettings () -> getHPBarSizeY ());
}

void battleMain :: drawTimerBar (BITMAP * buffer, int elapsedTime)
{
  //pic of clock next to it
  stretch_sprite (buffer, clockPic, 
		  dO -> getVideoSettings () -> getHPBarStartX () - dO -> getVideoSettings () -> getHPBarSizeY (), 
		  dO -> getVideoSettings () -> getArenaPosY () + fixtoi (dO -> getArena () -> getArenaSizeY () * dO -> getVideoSettings () -> getArenaScale ()) + int(.5 * dO -> getVideoSettings () -> getHPBarSizeY ()),
		  int(.5 * dO -> getVideoSettings () -> getHPBarSizeY ()), 
		  int(.5 * dO -> getVideoSettings () -> getHPBarSizeY ()));

  textprintf_ex(buffer, font, 
		dO -> getVideoSettings () -> getHPBarStartX () + 10, 
		dO -> getVideoSettings () -> getArenaPosY () + fixtoi (dO -> getArena () -> getArenaSizeY () * dO -> getVideoSettings () -> getArenaScale ()) + int(.5 * dO -> getVideoSettings () -> getHPBarSizeY ()), 
		makecol (128, 128, 128), -1, "%d", elapsedTime);
}

void battleMain :: drawStatus (BITMAP * buffer, playerUnit * mainChar)
{
  videoSettings * vs = dO -> getVideoSettings ();
  arena * aren = dO -> getArena ();

  bool invicDraw = false;
  bool speedupDraw = false;
  bool speeddownDraw = false;
  bool nojuiceDraw = false;
  bool morejuiceDraw = false;
  bool stunDraw = false;
  bool dmgdownDraw = false;

  int invicDrawPos = 0;
  int morejuiceDrawPos = 1;
  int speedupDrawPos = 2;

  int stunDrawPos = 0;
  int nojuiceDrawPos = 1;
  int speeddownDrawPos = 2;
  int dmgdownDrawPos = 3;
  int picSize = 8;

  int iconBrightness = 0;

  BITMAP * tempBitmap = create_bitmap (vs -> getResolutionY ()/ picSize, vs -> getResolutionY ()/ picSize);
  clear_bitmap (tempBitmap);
  set_trans_blender(255, 255, 255, 255);

  if (mainChar -> getInvicTime () > 0)
    invicDraw = true;

  if (mainChar -> getStatusAilment (statusAilmentSpeedBoost) > 0)
    speedupDraw = true;

  if (mainChar -> getStatusAilment (statusAilmentMoreJuice) > 0)
    morejuiceDraw = true;

  if (invicDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (invicPic, tempBitmap, 0, 0, invicPic -> w, invicPic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentInvic) > 0)
	iconBrightness = (255 * mainChar -> getInvicTime () / mainChar -> getBiggestStatusTime (statusAilmentInvic));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () + fixtoi (aren -> getArenaSizeX () * vs -> getArenaScale ()), vs -> getArenaPosY () + (invicDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (speedupDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (speedUpPic, tempBitmap, 0, 0, speedUpPic -> w, speedUpPic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentSpeedBoost) > 0)
	iconBrightness = (255 * mainChar -> getStatusAilment (statusAilmentSpeedBoost) / mainChar -> getBiggestStatusTime (statusAilmentSpeedBoost));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () + fixtoi (aren -> getArenaSizeX () * vs -> getArenaScale ()), vs -> getArenaPosY () + (speedupDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (morejuiceDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (morejuicePic, tempBitmap, 0, 0, morejuicePic -> w, morejuicePic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentMoreJuice) > 0)
	iconBrightness = (255 * mainChar -> getStatusAilment (statusAilmentMoreJuice) / mainChar -> getBiggestStatusTime (statusAilmentMoreJuice));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () + fixtoi (aren -> getArenaSizeX () * vs -> getArenaScale ()), vs -> getArenaPosY () + (morejuiceDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (mainChar -> getStunTime () > 0)
    stunDraw = true;

  if (mainChar -> getStatusAilment (statusAilmentNoJuice) > 0)
    nojuiceDraw = true;

  if (mainChar -> getStatusAilment (statusAilmentLessDamage) > 0)
    dmgdownDraw = true;

  if (mainChar -> getStatusAilment (statusAilmentSpeedSlow) > 0)
    speeddownDraw = true;

  if (stunDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (stunnedPic, tempBitmap, 0, 0, stunnedPic -> w, stunnedPic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentStun) > 0)
	iconBrightness = (255 * mainChar -> getStunTime () / mainChar -> getBiggestStatusTime (statusAilmentStun));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () - (vs -> getResolutionY ()/ picSize), vs -> getArenaPosY () + (stunDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (nojuiceDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (nojuicePic, tempBitmap, 0, 0, nojuicePic -> w, nojuicePic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentNoJuice) > 0)
	iconBrightness = (255 * mainChar -> getStatusAilment (statusAilmentNoJuice) / mainChar -> getBiggestStatusTime (statusAilmentNoJuice));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () - (vs -> getResolutionY ()/ picSize), vs -> getArenaPosY () + (nojuiceDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (speeddownDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (speedDownPic, tempBitmap, 0, 0, speedDownPic -> w, speedDownPic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentSpeedSlow) > 0)
	iconBrightness = (255 * mainChar -> getStatusAilment (statusAilmentSpeedSlow) / mainChar -> getBiggestStatusTime (statusAilmentSpeedSlow));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () - (vs -> getResolutionY ()/ picSize), vs -> getArenaPosY () + (speeddownDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  if (dmgdownDraw)
    {
      clear_bitmap (tempBitmap);
      stretch_blit (dmgDownPic, tempBitmap, 0, 0, dmgDownPic -> w, dmgDownPic -> h, 0, 0, tempBitmap -> w, tempBitmap -> h);

      if (mainChar -> getBiggestStatusTime (statusAilmentLessDamage) > 0)
	iconBrightness = (255 * mainChar -> getStatusAilment (statusAilmentLessDamage) / mainChar -> getBiggestStatusTime (statusAilmentLessDamage));
      else
	iconBrightness = 0;

      draw_lit_sprite(buffer, tempBitmap, vs -> getArenaPosX () - (vs -> getResolutionY ()/ picSize), vs -> getArenaPosY () + (dmgdownDrawPos * vs -> getResolutionY ()/ picSize), iconBrightness);
    }

  destroy_bitmap (tempBitmap);
}

void battleMain :: drawUnits (BITMAP * buffer)
{
  vector<unit *>::iterator the_iterator;

  the_iterator = units -> getUnits () -> begin();

  while( the_iterator < units -> getUnits () -> end() ) {    
    (*the_iterator) -> draw (buffer, gamePaused);
    the_iterator++;
  }
}

/////////////////////////////////////

  //////////////////////
 // MODEL PROCESSING //
//////////////////////

void battleMain :: applyInput ()
{
  //tell each unit to figure out what it wants to do next.

  //the player's unit should accept input.
  
  vector<unit *>::iterator the_iterator;

  the_iterator = units -> getUnits () -> begin();

  while(the_iterator != units -> getUnits () -> end() ) 
    {
      (*the_iterator) -> processInput (dO, units);
      the_iterator++;
    }

  //make pending actions.
  the_iterator = units -> getUnits () -> begin();

  while( the_iterator != units -> getUnits () -> end() ) 
    {
      (*the_iterator) -> makePendingActions (dO, units);
      the_iterator++;
    }

  //process pending actions.
  the_iterator = units -> getUnits () -> begin();

  while( the_iterator != units -> getUnits () -> end() ) 
    {
      (*the_iterator) -> processPendingActions (dO, units);
      the_iterator++;
    }

  //check the board for units to create!
  units -> createUnits ();
}

battleMain :: ~ battleMain ()
{
   destroy_bitmap(buffer);
   destroy_bitmap(gameOutput);
   destroy_bitmap(youWin);
   destroy_bitmap(youLose);
   destroy_bitmap(introPic);
}

#endif
