#ifndef _DATAOBJ_CPP_
#define _DATAOBJ_CPP_

#include <stdio.h>
#include <math.h>

#include "gameConstants.h"
#include "allegro.h"
#include "inputObj.h"
#include "videoSettings.h"
#include "arena.h"
#include "hitbox.h"

#include "dataObj.h"
#include <fstream>
#include <iostream>

#include <vector>
using namespace std;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

dataObj :: dataObj ()
{
  inpObj = new inputObj ();
  vS = new videoSettings ();
  aren = new arena ();
  arenaNum = 0;
}

void dataObj :: readConfig ()
{
  //reads the configuration file (tasmConfig.txt) and reads parameters.
  //if there is no file, the objects load their own defaults.
  fstream configFile;
  string strRead;
  int intRead;
  int resx = 640, resy = 480;
  bool keychange = false;
  vector <int> * keyMapChange;
  keyMapChange = new vector <int> (keyNumberOfKeys, -1);

  //read config file; get size defaults
  
  configFile.open ("tasmConfig.txt", ios::in);
  
  if (configFile.is_open ())
    {
      while (!configFile.eof())
	{
	  configFile >> strRead;

	  if (strRead.compare ("resolution") == 0)
	    {
	      //resolution is the screen resolution. 640 = 640x480, 800 = 800x600, 1024=1024x768
	      configFile >> intRead;
	      
	      if (intRead == 800)
		{resx = 800; resy = 600;}
	      else if (intRead == 1024)
		{resx = 1024; resy = 768;}
	      else
		{resx = 640; resy = 480;}

	      //set it
	      vS -> setResolution (resx, resy);
	    }
	  else if (strRead.compare ("colorDepth") == 0)
	    {
	      //color depth. 24 or 32 are accepted values.
	      configFile >> intRead;
	      
	      if (intRead == 32)
		{vS -> setColorDepth (32);}
	      else
		{vS -> setColorDepth (24);}
	    }
	  else if (strRead.compare ("windowed") == 0)
	    {
	      //windowed application. 1 = yes, 0 = no
	      configFile >> intRead;
	      
	      if (intRead == 1)
		{vS -> setWindowed (true);}
	      else
		{vS -> setWindowed (false);}
	    }
	  else if (strRead.compare ("controlThreshold") == 0)
	    {
	      //aim control style. See dumbunit.cpp -> aimunit for more specifics
	      configFile >> intRead;

	      inpObj -> setControlThreshold (intRead);
	    }
	  //keychange. This alters one of the given keys based on Allegro's scancode format.
	  else if (strRead.compare ("keyUp") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyUp) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyDown") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyDown) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyLeft") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyLeft) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyRight") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyRight) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyDefense") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyDefense) = intRead;
	      keychange = true;
	    }
	  else if (strRead.compare ("keyMenu") == 0)
	    {
	      configFile >> intRead;
	      keyMapChange -> at (keyMenu) = intRead;
	      keychange = true;
	    }
	}
      configFile.close ();
      if (keychange)
	inpObj -> setKeyMap (keyMapChange);
    }
}

//   videoSettings * vS;
videoSettings * dataObj :: getVideoSettings ()
{
  return vS;
}

int dataObj :: setVideoSettings (videoSettings * v)
{
  vS = v;
  return 0;
}

//   inputObj * inpObj; 
inputObj * dataObj :: getInputObj ()
{
  return inpObj;
}

int dataObj :: setInputObj (inputObj * iO)
{
  inpObj = iO;
  return 0;
}

//   arena * aren; 
arena * dataObj :: getArena ()
{
  return aren;
}

int dataObj :: setArena (arena * t)
{
  aren = t;
  return 0;
}

//DATAFILE * picData;

DATAFILE * dataObj :: getDatafile ()
{
  return picData;
}

DATAFILE * dataObj :: setDatafile (const char * filename)
{
  picData = load_datafile (filename);
  return picData;
}

dataObj :: ~dataObj ()
{
  unload_datafile (picData);
}

//////////////////////////////////////////  END OF dataObj
#endif
