#ifndef _DUMBUNIT_CPP
#define _DUMBUNIT_CPP_

#include "unitList.h"
#include "dumbUnit.h"

#include "allegro.h"
#include "dataObj.h"
#include "unit.h"
#include "unitGraphics.h"
#include "inputObj.h"
#include "cursor.h"
#include "gameConstants.h"
#include "tasmMath.h"

#include <vector>
using namespace std;

dumbUnit :: dumbUnit ()
{

}

void dumbUnit :: init (int param1, int param2, int param3)
{
  hpToGivePlayer = 0;
  tempVar1 = 0;
  polosionCount = 0;
  switch (unitType)
    {
    case (unitPoleonCircle):
      moveSpeed = itofix (20);
      moveType = moveTypeStuckToOwner;
      hitPoints = 2000;
      maxHP = 1000;
      timeToLive = 120;
      immuneToAilments = true;
      ignoreWall = true;
      affiliation = affiliationNeutral;
      break;
    case (unitPoleonShot):
      moveSpeed = itofix (20);
      moveType = moveTypeNormal;
      timeToLive = 30;
      affiliation = affiliationNeutral;
      immuneToAilments = true;
      break;
    case (unitPolosion):
      moveType = moveTypeDirect;
      hitPoints = 30000;
      timeToLive = 20;
      immuneToAilments = true;
      break;
    case (unitJuiceBox):
      moveType = moveTypeDirect;
      hitPoints = 1;
      timeToLive = 210;
      immuneToAilments = true;
      diesWhenHitsAnything = true;
      helpsPlayer = true;
      break;
    case (unitPlayerAim):
      moveType = moveTypeStuckToOwner;
      hitPoints = 20000;
      maxHP = 30000;
      statusAilment -> at (statusAilmentInvic) = 30000;
      immuneToAilments = true;
      ignoreWall = true;
      affiliation = affiliationNeutral;
      aimAngle = itofix (0);
      break;
    default:
      break;
    }
}

void dumbUnit :: processInput (dataObj * dO, unitList * units)
{
  vector <unit*> * gub = NULL;
  int numPC = 0;
  int cursorX = 0, cursorY = 0;
  unit * ownUnit = NULL;
  fixed micX = itofix (0), micY = itofix (0), micDist = itofix (0), micAngle = itofix (0), turnDist = itofix (0);
  int ct = dO -> getInputObj () -> getControlThreshold ();

  switch (unitType)
    {
    case (unitPoleonCircle):

      gub = units -> search (units -> getUnits (), searchByUnitType, unitPoleonCircle);
      gub = units -> search (gub, searchByOwner, unitPoleon);
      numPC = gub -> size ();

      if (numPC == 1)
	{
	  if (unitID == gub -> at (0) -> getUnitID ())
	    tempVar1 = (tempVar1 + 4) ^ 256;
	}
      else if (numPC == 2)
	{
	  if (unitID == gub -> at (0) -> getUnitID ())
	    tempVar1 = (tempVar1 + 4) ^ 256;
	  if (unitID == gub -> at (1) -> getUnitID ())
	    tempVar1 = (gub -> at (0) -> tempVar1 + 128) ^ 256;
	}
      else if (numPC == 3)
	{
	  if (unitID == gub -> at (0) -> getUnitID ())
	    tempVar1 = (tempVar1 + 4) ^ 256;
	  if (unitID == gub -> at (1) -> getUnitID ())
	    tempVar1 = (gub -> at (0) -> tempVar1 + 85) ^ 256;
	  if (unitID == gub -> at (2) -> getUnitID ())
	    tempVar1 = (gub -> at (0) -> tempVar1 + 170) ^ 256;
	}
      drawAngle = itofix ((-tempVar1) ^ 256);
      offsetX = fixmul (itofix (50), fixsin (itofix (tempVar1)));
      offsetY = fixmul (itofix (50), fixcos (itofix (tempVar1)));

      break;
    case (unitPoleonShot):
      moveMag = 4;
      break;
    case (unitPolosion):
      offsetX = posX;
      offsetY = posY;
      drawScale = ftofix (0.03) * polosionCount;
      polosionCount++;
      break;
    case (unitJuiceBox):
      offsetX = posX;
      offsetY = posY;
      break;
    case (unitPlayerAim):
      //be affected by the mouse. This implementation may change, depending on the UI structure.
      
      //change angle to point at mouse. The arena's scaling is taken into account.
      dO -> getInputObj () -> getCursor () -> getMouseMickeys (cursorX, cursorY);
      //transformation for screen vs cartesian coordinates
      micX = itofix (-1 * cursorX);
      micY = itofix (cursorY);
      micDist = fixhypot (micX, micY);

      ownUnit = units -> at (getOwnerID ());

      if (ownUnit != NULL && micDist > itofix (1)) //insert threshold value here
	    {
	      //get angle
	      if (cursorY == 0 && cursorX < 0)
		micAngle = itofix (-64);
	      else if (cursorY == 0  && cursorX > 0)
		micAngle = itofix (64);
	      else
		micAngle = fixatan (fixdiv (micX, micY));

	      //correct the atan response: answers will only range from -64 to 64, crossing 0 (point up only)...need to rotate answer if it should be pointing down

	      if (micY > itofix (0))
		micAngle -= itofix (128);
	      
	      //keeps micAngle in the range of -256 to 256
	      micAngle = micAngle ^ 256;

	      //combine control stlye 1&2
	      //use fast mouse jerks to quickly get the angle
	      //move slowly to fine tune the angle
	      //NOTE: Need control threshold values to separate fast vs slow mouse movements
	      if (micDist > itofix (ct))
		{
		  //Idea #1: direct - make the cursor point in the direction the mouse moves. Will need to control the threshold value so when user releases mouse, doesn't turn around (might just want only fullscreen for this, since the mouse WILL scroll outside of the window.) Also, it's inaccurate.
		  drawAngle = micAngle;
		}
	      else
		{
		  //Idea #2: rotate - the mouse direction acts as a rotation force to 'rotate' the cursor around. It may be unintuitive for large rotations.
		  //ok, compare the angles
		  turnDist = angleDist (micAngle, drawAngle);
		  //ok rotate based on the difference between the angles
		  drawAngle += turnDist / 10;
		}
	    }
      break;
    }
}

dumbUnit :: ~dumbUnit ()
{

}
#endif

/* OLD CODE FOR MOUSE POINT IMPLEMENTATION
  fixed cursorDistX = itofix (0), cursorDistY = itofix (0), cursorDist = itofix (0), cursorAngle = itofix (0);
  fixed newCursorX = itofix (0), newCursorY = itofix (0);
  fixed aimerSize = itofix (1);

....

      //keep the mouse withing the aiming ring. Also, move the mouse with this unit so the aim is consistent.
      if (ownUnit != NULL)
	{
	  if (getUnitGraphics () != NULL)
	    {
	      aimerSize = (getUnitGraphics () -> getPicSize ()) * (dO -> getVideoSettings () -> getArenaScale () / 2);
	    }

	  cursorAngle = calcAngle (itofix (dO -> getVideoSettings () -> getArenaPosX ()) + (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosX ()),
				 itofix (dO -> getVideoSettings () -> getArenaPosY ()) + (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosY ()), 
				 itofix (cursorX), 
				 itofix (cursorY), 
				 ownUnit -> getDrawAngle ());

	  drawAngle = cursorAngle;

	  //curtail the mouse so it doesn't move too far away from the unit (seems like sluggish input, bad for an action game!)

	  cursorDist = calcDist (itofix (dO -> getVideoSettings () -> getArenaPosX ()) + (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosX ()),
				 itofix (dO -> getVideoSettings () -> getArenaPosY ()) + (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosY ()), 
				 itofix (cursorX), 
				 itofix (cursorY));

	  cursorDistX = itofix (cursorX) - itofix (dO -> getVideoSettings () -> getArenaPosX ()) - (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosX ());

	  cursorDistY = itofix (cursorY) - itofix (dO -> getVideoSettings () -> getArenaPosY ()) - (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosY ());

	  cursorDist = fixhypot (cursorDistX, cursorDistY);

	  if (cursorDist > aimerSize)
	    {
	      //scale the mouse cursor so it's at the same angle but at aimerSize distance.
	      newCursorX = fixmul (aimerSize, fixdiv (cursorDistX, cursorDist));
	      newCursorY = fixmul (aimerSize, fixdiv (cursorDistY, cursorDist));
	    }

	  cursorX = dO -> getVideoSettings () -> getArenaPosX () + fixtoi (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosX ()) + fixtoi (newCursorX);
	  cursorY = dO -> getVideoSettings () -> getArenaPosY () + fixtoi (dO -> getVideoSettings () -> getArenaScale () * ownUnit -> getDrawPosY ()) + fixtoi (newCursorY);
	  dO -> getInputObj () -> getCursor () -> setCursorShouldBe (cursorX, cursorY);
}
*/
