#ifndef _INPUTOBJ_CPP_
#define _INPUTOBJ_CPP_

#include <stdio.h>
#include <math.h>

#include "allegro.h"
#include "cursor.h"
#include "gameConstants.h"

#include "inputObj.h"

#include <vector>
using namespace std;

inputObj ::  inputObj ()
{
  keyMap = new vector <int> (keyNumberOfKeys, KEY_F1);
  keyWasHit = new vector <bool> (keyNumberOfKeys, false);
  keyWasClicked = new vector <bool> (keyNumberOfKeys, false);
  keyMap -> at (keyUp) = keyBindUp;
  keyMap -> at (keyDown) = keyBindDown;
  keyMap -> at (keyLeft) = keyBindLeft;
  keyMap -> at (keyRight) = keyBindRight;
  keyMap -> at (keyDefense) = keyBindDefense;
  keyMap -> at (keyMenu) = keyBindMenu;
  keyMap -> at (keyPause) = keyBindPause;
  keyMap -> at (keyQuit) = keyBindQuit;
  mouseWasHit = new vector <bool> (3, false);
  mouseWasClicked = new vector <bool> (3, false);
  cur = new cursor ();
  controlThreshold = 0;
}

void inputObj :: processInput (int resX, int resY, bool readyToClear)
{
  int i = 0, iMax = 0;
  int mouseMickeyX = 0, mouseMickeyY = 0;
  int curSX, curSY;
  cur -> getCursorShouldBe (curSX, curSY);

  iMax = keyNumberOfKeys;

  //should be detecting release events, not pressed (and held down) events

  for (i = 0; i < iMax; i ++)
    {
      if (keyWasHit -> at (i) && !key[keyMap -> at (i)])
	{
	  keyWasClicked -> at (i) = true;
	}
      else
	{
	  if (readyToClear)
	    keyWasClicked -> at (i) = false;
	}

      keyWasHit -> at(i) = key[keyMap -> at (i)];
      if (keyWasHit -> at(i))
	setMouseUsedLast (false);
    }

  //detect mouse stuff
  get_mouse_mickeys (&mouseMickeyX, &mouseMickeyY);

  if ((mouseMickeyX != 0) || (mouseMickeyY != 0))
    setMouseUsedLast (true);

  cur -> setMouseMickeys (mouseMickeyX, mouseMickeyY);

  curSX = mouse_x;
  curSY = mouse_y;

  if (curSX < 0)
    curSX = 0;

  if (curSY < 0)
    curSY = 0;

  if (curSX > resX)
    curSX = resX;

  if (curSY > resY)
    curSY = resY;

  cur -> setCursorShouldBe (curSX, curSY);

  if (mouseWasHit -> at (0) && !(mouse_b & 1))
    mouseWasClicked -> at (0) = true;
  else if (readyToClear)
    mouseWasClicked -> at (0) = false;
  if (mouse_b & 1)
    {
      mouseWasHit -> at(0) = true;
      setMouseUsedLast (true);
    }
  else
    mouseWasHit -> at(0) = false;

  if (mouseWasHit -> at (1) && !(mouse_b & 2))
    mouseWasClicked -> at (1) = true;
  else if (readyToClear)
    mouseWasClicked -> at (1) = false;
  if (mouse_b & 2)
    {
      mouseWasHit -> at(1) = true;
      setMouseUsedLast (true);
    }
  else
    mouseWasHit -> at(1) = false;

  if (mouseWasHit -> at (2) && !(mouse_b & 4))
    mouseWasClicked -> at (2) = true;
  else if (readyToClear)
    mouseWasClicked -> at (2) = false;
  if (mouse_b & 4)
    {
      mouseWasHit -> at(2) = true;
      setMouseUsedLast (true);
    }
  else
    mouseWasHit -> at(2) = false;
}

///////////////////// Acessors and Modifiers
//  vector<int> * keyMap;
vector<int>* inputObj :: getKeyMap ()
{
  return keyMap;
}

int inputObj :: setKeyMap (vector<int>* keyMap2)
{
  //this replaces the key mappings of keyMap with keyMap2.
  //keyMap2 must be of the same length as keyMap.
  //an entry in keyMap will NOT be overwritten if the entry in keyMap2 is -1.
  int ke;

  //keyMap = keyMap2;
  if (keyMap -> size () > keyMap2 -> size ())
    return -1;

  for (int i = 0; i < int (keyMap -> size ()); i ++)
    {
      ke = keyMap2 -> at (i);
      if (ke != -1)
	keyMap -> at (i) = ke;
    }
  return 0;
}

//  vector<bool> * keyWasHit;
vector<bool>* inputObj :: getKeyWasHit ()
{
  return keyWasHit;
}

int inputObj :: setKeyWasHit (vector<bool>* kwh)
{
  keyWasHit = kwh;
  return 0;
}

//  vector<bool> * keyWasClicked;
vector<bool>* inputObj :: getKeyWasClicked ()
{
  return keyWasClicked;
}

int inputObj :: setKeyWasClicked (vector<bool>* kwc)
{
  keyWasClicked = kwc;
  return 0;
}

//  vector<bool> * mouseWasHit;
vector<bool>* inputObj :: getMouseWasHit ()
{
  return mouseWasHit;
}

int inputObj :: setMouseWasHit (vector<bool>* mwh)
{
  mouseWasHit = mwh;
  return 0;
}

//  vector<bool> * mouseWasClicked;
vector<bool>* inputObj :: getMouseWasClicked ()
{
  return mouseWasClicked;
}

int inputObj :: setMouseWasClicked (vector<bool>* mwc)
{
  mouseWasClicked = mwc;
  return 0;
}

//  int inputWait;
int inputObj :: getInputWait ()
{
  return inputWait;
}

void inputObj :: setInputWait (int iW)
{
  inputWait = iW;
}


//  bool mouseUsedLast;
void inputObj :: getMouseUsedLast (bool & mUL)
{
  mUL = mouseUsedLast;
}

void inputObj :: setMouseUsedLast (bool mUL)
{
  mouseUsedLast = mUL;
}

//  bool keyWasHitThisFrame;
void inputObj :: getKeyWasHitThisFrame (bool & keyWasHitTF)
{
  keyWasHitTF = keyWasHitThisFrame;
}

void inputObj :: setKeyWasHitThisFrame (bool keyWasHitTF)
{
  keyWasHitThisFrame = keyWasHitTF;
}

//  cursor * cur;
cursor * inputObj :: getCursor ()
{
  return cur;
}

int inputObj :: setCursor (cursor * cur2)
{
  cur = cur2;
  return 0;
}

void inputObj :: setControlThreshold (int cs)
{
  controlThreshold = cs;
}

int inputObj :: getControlThreshold ()
{
  return controlThreshold;
}

inputObj ::  ~inputObj ()
{

}
#endif
