#ifndef _UNIT_CPP_
#define _UNIT_CPP_

#include <stdio.h>

#include "unitCreate.h"
#include "unitList.h"

#include "unit.h"

#include "allegro.h"

#include "dataObj.h"
#include "unitGraphics.h"
#include "gameConstants.h"
#include "hitbox.h"
#include "tasmMath.h"

#include <vector>
using namespace std;

unit :: unit ()
{
  posX = itofix (400), posY = itofix (300);
  drawX = 400, drawY = 300;
  drawAngle = 0;
  drawScale = ftofix (1.0);
  moveAngle = 0;
  moveMag = 0;
  knockbackDrift = 0;
  numHitsWhileStun = 0;
  cantAttackTime1 = 0;
  cantAttackTime2 = 0;
  cantAttackTime3 = 0;
  cantMoveTime = 0;
  affiliation = affiliationNeutral;

  knockbackX = itofix (0);
  knockbackY = itofix (0);
  knockbackMul = ftofix (1.0);

  unitID = -1;
  unitType = unitNull;
  ownerID = -1;

  picFrame = 0;
  currentActivity = charStatusIdle;
  pendingActivity = charStatusIdle;
  wantsToTaunt = false;

  moveSpeed = itofix (5);
  moveType = moveTypeNormal;
  offsetX = 0;
  offsetY = 0;

  hb = NULL;
  uG = NULL;

  pendingAttack = 0;
  currentAttack = 0;

  hitPoints = 1;
  pendingHPloss = 0;
  maxHP = 30000;
  hpToGivePlayer = 5;
  deathTimeRemoval = 10;
  timeToLive = -256; //any value < -10 means it will not decay; must recieve damage to die.
  immuneToAilments = false;
  helpsPlayer = false;

  diesWhenHitsWall = false;
  hitsWall = false;
  ignoreWall = false;
  statusAilment = new vector <int> (numStatusAilment, 0);
  biggestStatusTime = new vector <int> (numStatusAilment, 1);
  diesWhenHitsAnything = false;
  hitSomething = false;

  tempVar1 = 0;
}

void unit :: init (int param1, int param2, int param3)
{
  //this is useful for passing custom parameters to the unit. Each unit decides what it wants to do with the parameters.
}

void unit :: draw (BITMAP * buffer, bool gamePaused)
{
  //draws the unit (using the unitGraphics and its frame (picFrame)) on the buffer; Also sets pending status into current status.

  //translate coordinates into drawing coordinates
  drawX = fixtoi (posX);
  drawY = fixtoi (posY);

  //get next frame...if the game isn't paused
  BITMAP * pic = NULL;

  if (!gamePaused)
    {
      if (pendingActivity != currentActivity)
	{
	  currentActivity = pendingActivity;
	  picFrame = 0;
	}

      pic = uG -> getPic (currentActivity, picFrame);
      picFrame ++;
    }
  else
    pic = uG -> getPic (currentActivity, picFrame);

  if (picFrame >= uG -> getSizeOfVector (currentActivity))
    {
      if (((currentActivity >= charStatusAttack1 &&
	  currentActivity <= charStatusAttack3) ||
	   currentActivity == charStatusDefense) && currentAttack != 0 && cantAttackTime1 <= 0 && cantAttackTime2 <= 0 && cantAttackTime3 <= 0)

	{
	  currentActivity = charStatusIdle;
	  currentAttack = 0;
	}
      picFrame = 0;
      wantsToTaunt = false;
    }

  if (pic != NULL)
    pivot_scaled_sprite (buffer, 
			 pic,
			 drawX, drawY, 
			 uG -> getPicSize() / 2,
			 uG -> getPicSize() / 2,
			 drawAngle, drawScale);
    /*
    pivot_sprite (buffer, 
		  pic,
		  drawX, drawY, 
		  uG -> getPicSize() / 2,
		  uG -> getPicSize() / 2,
		  drawAngle);
    */
}

void unit :: processInput (dataObj * dO, unitList * units)
{
  //accepts input. Player-controlled units read inputObj, while cpu-controlled units do the AI dance.
}


void unit :: makePendingActions (dataObj * dO, unitList * units)
{
  //the unit 1) checks its status (dead, stun, etc). 2) checks for collisions


  //check status based on what the user desires i.e. does the player want to attack or move?
  statusCheck (dO, units);
    
  //clearing knockback (unless knockbackdrift is still active)
  if (knockbackDrift <= 0)
    {
      knockbackX = itofix (0);
      knockbackY = itofix (0);
    }
  
  //check for collisions and calculate the total knockback, and damage
  collideCheck (dO, units);

  if (knockbackDrift > 30)
    knockbackDrift = 30;

  if (knockbackDrift > 0)
    knockbackDrift--;
}

void unit :: processPendingActions (dataObj * dO, unitList * units)
{
  //after calculating everything, apply the results: 1) damage, 2) juice for the player, 3)move the unit

  fixed tempPosX = posX;
  fixed tempPosY = posY;
  fixed kbc, kbdist;
  float speedBoost = 0.25;
  unit * owningUn = NULL;

  //lose HP
  hitPoints -= pendingHPloss;

  //giving 'juice' to the player
  vector<unit *> * playerUnit = units -> search (units -> getUnits (), searchByAffiliation, affiliationPlayer);

  if (playerUnit -> size () > 0)
    {
      if (pendingActivity != charStatusDead && hitPoints <= 0)
	giveJuiceToPlayer (playerUnit -> at (0), hpToGivePlayer, 2); //give juice when defeated
      else if (pendingActivity != charStatusStun && getStatusAilment (statusAilmentStun) > 0 && hitPoints > 0)
	giveJuiceToPlayer (playerUnit -> at (0), hpToGivePlayer, 5); //give juice when stunned
      else if (pendingHPloss > 0 && hitPoints > 0)
	giveJuiceToPlayer (playerUnit -> at (0), hpToGivePlayer, 1); //give juice when hit
    }
  //you can't exceed your max hp
  if (hitPoints > maxHP)
    hitPoints = maxHP;

  //if the player was damaged, give him invicibility (30 per second)
  if (pendingHPloss > 0 && getAffiliation () == affiliationPlayer)
    statusAilment -> at (statusAilmentInvic) = 15;
  
  if (statusAilment -> at (statusAilmentInvic) > 0)
    {
      statusAilment -> at (statusAilmentInvic)--;
    }

  if (statusAilment -> at (statusAilmentInvic) <= 0)
    {
      biggestStatusTime -> at (statusAilmentInvic) = 1;
    }
  
  if (statusAilment -> at (statusAilmentInvic) > biggestStatusTime -> at (statusAilmentInvic))
    biggestStatusTime -> at (statusAilmentInvic) = statusAilment -> at (statusAilmentInvic);

  pendingHPloss = 0;

  //checking status ailments
  for (int i = 0; i < int (statusAilment -> size ()); i ++)
    {
      if (statusAilment -> at (i) > 0)
	{
	  statusAilment -> at (i) --;
	}

      if (statusAilment -> at (i) <= 0)
	{
	  biggestStatusTime -> at (i) = 1;
	}
    }

  //add speed modifiers based on status ailments
  if (statusAilment -> at (statusAilmentSpeedBoost) > 0)
    speedBoost *= 1.5;

  if (statusAilment -> at (statusAilmentSpeedSlow) > 0)
    speedBoost /= 2.0;

  //apply unit movement based on moveType
  if (moveType == moveTypeNormal)
    {
      //normal: move moveSpeed units at moveAngle. moveAngle follows Allegro's angle system -- 256 degrees, starting with 0 at the top. speedBoost is affected by status boosts (and the 0.25 normalizer) and moveMag (set by the AI.)
      tempPosX += fixmul (fixsin (moveAngle), fixmul (moveSpeed, ftofix (speedBoost * moveMag)));
      tempPosY -= fixmul (fixcos (moveAngle), fixmul (moveSpeed, ftofix (speedBoost * moveMag)));
    }
  else if (moveType == moveTypeDirect)
    {
      //direct: stay in place; the offset is controlled by the AI to follow a direct path un obstructed by obstacles.
      tempPosX = offsetX;
      tempPosY = offsetY;
    }
  else if (moveType == moveTypeStuckToOwner)
    {
      //stuck to owner (or the owner's last place.) Offset is controlled by the unit's AI.
      owningUn = units -> getUnit (ownerID);
      
      if (owningUn != NULL)
	{
	  tempPosX = owningUn -> getPosX () + offsetX;
	  tempPosY = owningUn -> getPosY () + offsetY;
	}
    }

  //add knockback, after capping its maximum (don't want unit to skip all over the place)
  kbc = dO -> getArena () -> getKnockbackCap ();
  kbdist = fixhypot (knockbackX, knockbackY);
  if (kbdist > kbc)
    {
      knockbackX = fixmul (fixmul (knockbackX, fixdiv (kbc, kbdist)), knockbackMul);
      knockbackY = fixmul (fixmul (knockbackY, fixdiv (kbc, kbdist)), knockbackMul);
    }

  tempPosX += knockbackX;
  tempPosY += knockbackY;

  //check the new position to make sure it isn't a wall
  arenaBoundaryCheck (tempPosX, tempPosY, uG, dO);
  
  //finally, confirm the final position
  posX = tempPosX;
  posY = tempPosY;
}

void unit :: addAttacks (dataObj * dO, unitList *, int attack)
{
  //creates an attack, and adds it to unitList to be created.
  //this is done by the AI.
  //as a dummy unit, this won't do anything.
}

//auxillary funcions
void unit :: collideCheck (dataObj * dO, unitList * units)
{
  //collideCheck is huge, but simple. This unit checks its hitbox against all other hitboxes. Depending on the type of collision, it applies knockback and/or damage.

  int cA;
  hitbox * hbTemp = NULL;
  fixed hbDist = itofix (0);
  fixed angleCollide;
  fixed magCollide;
  bool selfStunned = false;
  bool targetStunned = false;
  bool deathCheck = true;
  float statusAilmentKnockbackMod = 1.0;
  float statusAilmentDamageMod = 1.0;
  unit * ownerUnit = NULL;
  float comboBuffer = 1.0;

  vector<unit *>::iterator the_iterator;
  vector<unit *> * unitsToCheck = units -> getUnits ();

  //look at units
  the_iterator = unitsToCheck -> begin();

  if (currentActivity == charStatusStun)
    selfStunned = true;
  else
    selfStunned = false;
  
  while( the_iterator < unitsToCheck -> end() ) {
    targetStunned = false;
    deathCheck = true;
    statusAilmentKnockbackMod = 1.0;
    statusAilmentDamageMod = 1.0;
    comboBuffer = 1.0;
    hbTemp = NULL;
    hbDist = itofix (30000);
    ownerUnit = units -> getUnit ((*the_iterator) -> getOwnerID ());

    if ((*the_iterator) -> currentActivity == charStatusStun)
      targetStunned = true;
    else
      targetStunned = false;
    
    cA = (*the_iterator) -> getHitbox () -> checkCollide (unitType, targetStunned);

    /*
    if (!targetStunned)
      cA = hb -> checkCollide ((*the_iterator) -> getUnitType (), selfStunned);
    else
    */

    //deathCheck:
    if (getAffiliation() == affiliationPlayer && (*the_iterator) -> getActivity () == charStatusDead && !((*the_iterator) -> getHelpsPlayer ()))
      {
	deathCheck = false;
      }

    //let's reduce the number of collision checks. Don't check if: 
    // the unit cannot bump into itself.
    // the collisionAction between these types of objects is to ignore
    // this is player and the target is dead (deathCheck)

    if ((*the_iterator) -> getUnitID () != unitID && cA != collideIgnore && deathCheck && deathCheck)
      {
	//compare distances to centers. 
	hbTemp = (*the_iterator) -> getHitbox ();
	if (hbTemp != NULL)
	  hbDist = calcDist (posX, posY,
			     (*the_iterator) -> getPosX (),
			     (*the_iterator) -> getPosY ());
	
	//If too close,
	if (hbDist <= ((uG -> getPicSize () * drawScale) + ((*the_iterator) -> getUnitGraphics () -> getPicSize () * (*the_iterator) -> getDrawScale ()))/2)
	  {
	    //the results depend on the type of collision.
	    //collision applies knockback so they are not touching each other, and it deals damage. knockbackonly and damageonly use only one of those two consequences.

	    //calculate comboBuffer here
	    //Less effort is awarded for multiple hits while the enemy is stunned. This is called a 'combo buffer' in most fighting games.
	    //While the enemy is stunned, each hit's effectiveness is at 70% of its full power. 
	    for (int temp = 0; temp < numHitsWhileStun; temp++)
	      comboBuffer *= .7;
	    
	    if (cA == collideBoth || cA == collideKnockbackOnly)
	      {
		//apply hitbox
		angleCollide = calcAngle (posX, posY,
					  (*the_iterator) -> getPosX (),
					  (*the_iterator) -> getPosY (),
					  moveAngle);
		
		magCollide = calcDist (posX, posY,
				       (*the_iterator) -> getPosX (),
				       (*the_iterator) -> getPosY ());
		
		if ((*the_iterator) -> getHitbox () -> knockbackType == knockbackRelative | (*the_iterator) -> getHitbox () -> knockbackType == knockbackAbsolute)
		  {
		    knockbackX -= fixmul (ftofix (comboBuffer), fixmul (fixsin (angleCollide), magCollide));
		    knockbackY += fixmul (ftofix (comboBuffer), fixmul (fixcos (angleCollide), magCollide));
		  }
		
		if ((*the_iterator) -> getHitbox () -> knockbackType == knockbackAbsolute | (*the_iterator) -> getHitbox () -> knockbackType == knockbackAbsoluteOnly)
		  {
		    if (ownerUnit != NULL && ownerUnit -> getStatusAilment (statusAilmentLessDamage) > 0)
		      statusAilmentKnockbackMod = 0.1;
		    
		    
		    knockbackX -= fixmul (fixsin (angleCollide + (*the_iterator) -> getHitbox () -> knockbackAbsAngle), int ((*the_iterator) -> getHitbox () -> knockbackAbsMag * statusAilmentKnockbackMod));
		    knockbackY += fixmul (fixcos (angleCollide + (*the_iterator) -> getHitbox () -> knockbackAbsAngle), int ((*the_iterator) -> getHitbox () -> knockbackAbsMag * statusAilmentKnockbackMod));
		  }
		//add knockbackdrift; effectively ignored with knockbackdrift
		knockbackDrift += int (hbTemp -> knockbackDrift * statusAilmentKnockbackMod * comboBuffer * comboBuffer * comboBuffer );
	      }
	    if (cA == collideBoth || cA == collideDamageOnly)
	      {
		if (ownerUnit != NULL && ownerUnit -> getStatusAilment (statusAilmentLessDamage) > 0)
		  statusAilmentDamageMod = 0.5;

		if (statusAilment -> at (statusAilmentInvic) <= 0)
		  {
		    hitSomething = true;
		    pendingHPloss += int ((*the_iterator) -> getHitbox () -> damageDealt * statusAilmentDamageMod * comboBuffer);
		  }

		if (!immuneToAilments)
		  {
		    //add status ailments (stun is now part of it!)

		    //deal damage, status ailments, and stun.

		    if (statusAilment -> at (statusAilmentInvic) <= 0)
		      {
			statusAilment -> at (statusAilmentNoJuice) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentNoJuice) * comboBuffer);
			statusAilment -> at (statusAilmentLessDamage) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentLessDamage) * comboBuffer);
			statusAilment -> at (statusAilmentStun) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentStun) * comboBuffer);
			statusAilment -> at (statusAilmentSpeedSlow) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentSpeedSlow) * comboBuffer);
		      }

		    statusAilment -> at (statusAilmentSpeedBoost) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentSpeedBoost) * comboBuffer);
		    statusAilment -> at (statusAilmentInvic) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentInvic) * comboBuffer);
		    statusAilment -> at (statusAilmentMoreJuice) +=  int ((*the_iterator) -> getHitbox () -> statusAilmentDmg -> at (statusAilmentMoreJuice) * comboBuffer);


		    //add to the number of hits while stunned; this factors into the combo buffer
		    numHitsWhileStun++;

		    //now, for the biggest ailment time...
		    for (int i = 0; i < int (statusAilment -> size ()); i++)
		      if (statusAilment -> at (i) > biggestStatusTime -> at (i))
			biggestStatusTime -> at (i) = statusAilment -> at (i);
		  }
	      }
	  }
      }
    the_iterator++;
  }
}

void unit :: statusCheck (dataObj * dO, unitList * units)
{
  //determines the pendingActivity of the unit. The draw function makes the pendingActivity current. This is done to prevent race conditions during hit detection.
  //here's the order of status determination: dead, stun, attack, move, defense, idle
  //this also decrements attack cooldown times.

  timeToLive --;

  if (hitPoints <= 0 || (diesWhenHitsWall && hitsWall && !ignoreWall) || (diesWhenHitsAnything && hitSomething) || (timeToLive < 0 && timeToLive > -10)) 
    {
      //you are dead when:
      //1) you run out of hitPoints.
      //2) you hit a wall and you're supposed to die when you hit a wall.
      //3) your timeToLive has expired.
      //4) you hit something that deals damage (collide type both or damageOnly) and you're supposed to die when that happens.

      if (diesWhenHitsAnything && hitSomething)
	hitPoints = 0;

      if (diesWhenHitsWall && hitsWall && !ignoreWall)
	hitPoints = 0;

      if (diesWhenHitsWall && hitsWall && !ignoreWall)
	hitPoints = 0;

      if (timeToLive < 0 && timeToLive > -10 && hitPoints > 0)
	hitPoints = 0;

      pendingActivity = charStatusDead;
      deathTimeRemoval --;
    }
  else if (statusAilment -> at (statusAilmentStun) > 0) //check if stunned here
    {
      statusAilment -> at (statusAilmentStun) --;
      pendingActivity = charStatusStun;
      if (statusAilment -> at (statusAilmentStun) <= 0)
	{
	  biggestStatusTime -> at (statusAilmentStun) = 1;
	}
    }
  else if ((((pendingAttack & 1) > 0) && cantAttackTime1 <= 0) || 
	   (((pendingAttack & 2) > 0) && cantAttackTime2 <= 0) || 
	   (((pendingAttack & 4) > 0) && cantAttackTime3 <= 0))
    {
      //character is attacking.

      if ((pendingAttack & 1) > 0)
	pendingActivity = charStatusAttack1;

      if ((pendingAttack & 2) > 0)
	pendingActivity = charStatusAttack2;

      if ((pendingAttack & 4) > 0)
	pendingActivity = charStatusDefense;

      //add this attack to the unitList

      addAttacks (dO, units, pendingAttack);

      pendingAttack = 0;
    }
  else if (currentAttack != 0)
    {
      //character is attacking.
      if ((currentAttack & 1) > 0)
	pendingActivity = charStatusAttack1;

      if ((currentAttack & 2) > 0)
	pendingActivity = charStatusAttack2;

      if ((currentAttack & 4) > 0)
	pendingActivity = charStatusDefense;
    }
  else if (moveMag != 0 && cantMoveTime <= 0)
    {
      //character is moving.
      pendingActivity = charStatusRun;
    }
  else if (wantsToTaunt)
    {
      //character is taunting.
      pendingActivity = charStatusTaunt;
    }
  else if (currentActivity != charStatusTaunt)
    {
      //and is not doing anything else, character is idle.
      pendingActivity = charStatusIdle;
    }

  if (cantAttackTime1 > 0)
    cantAttackTime1 --;

  if (cantAttackTime2 > 0)
    cantAttackTime2 --;

  if (cantAttackTime3 > 0)
    cantAttackTime3 --;

  if (cantMoveTime > 0)
    cantMoveTime --;

  if (statusAilment -> at (statusAilmentStun) <= 0)
    numHitsWhileStun = 0;

  hitsWall = false;
  hitSomething = false;
}

void unit :: arenaBoundaryCheck (fixed & tempPosX, fixed & tempPosY, unitGraphics * uG, dataObj * dO)
{
  //This checks if the unit has gone beyond the arena's boundary.
  //If it has and it ignoreWall is false, it is forced back into the arena, and hitsWall is set to true.

  int arenaSizeX, arenaSizeY;

  if (ignoreWall)
    return;

  arenaSizeX = dO -> getArena () -> getArenaSizeX ();
  arenaSizeY = dO -> getArena () -> getArenaSizeY ();

  if (tempPosX < (0 + drawScale * uG -> getPicSize()/2))
    {
      tempPosX = (0 + drawScale * uG -> getPicSize()/2);
      knockbackX *= -1;
      hitsWall = true;
    }
  else if (tempPosX > itofix (arenaSizeX) - drawScale * uG -> getPicSize()/2)
    {
      tempPosX = itofix (arenaSizeX) - drawScale * uG -> getPicSize()/2;
      knockbackX *= -1;
      hitsWall = true;
    }
  
  if (tempPosY < 0 + drawScale * uG -> getPicSize()/2)
    {
      tempPosY = 0 + drawScale * uG -> getPicSize()/2;
      knockbackY *= -1;
      hitsWall = true;
    }
  else if (tempPosY > itofix (arenaSizeY) - drawScale * uG -> getPicSize()/2)
    { 
      tempPosY = itofix (arenaSizeY) - drawScale * uG -> getPicSize()/2;
      knockbackY *= -1;
      hitsWall = true;
    }
}

//accessors and mutators. Should be self-explanatory.
int unit :: getDrawPosX ()
{
  return drawX;
}

int unit :: getDrawPosY ()
{
  return drawY;
}

void unit :: setDrawPos (int x, int y)
{
  drawX = x;
  drawY = y;
}

void unit :: setPos (fixed x, fixed y)
{
  posX = x;
  posY = y;
}

fixed unit :: getPosX ()
{
  return posX;
}

fixed unit :: getPosY ()
{
  return posY;
}

fixed unit :: getDrawAngle ()
{
  return drawAngle;
}

fixed unit :: getMoveAngle ()
{
  return moveAngle;
}

void unit :: setDrawAngle (fixed da)
{
  drawAngle = da;
}

void unit :: setMoveAngle (fixed ma)
{
  moveAngle = ma;
}

int unit :: getDeathTimeRemoval ()
{
  return deathTimeRemoval;
}

void unit :: setDeathTimeRemoval (int gub)
{
  deathTimeRemoval = gub;
}

void unit :: setUnitGraphics (unitGraphics * ug2)
{
  uG = ug2;
}

unitGraphics * unit :: getUnitGraphics ()
{
  return uG;
}

void unit :: setHitbox (hitbox * hb2)
{
  hb = hb2;
}

hitbox * unit :: getHitbox ()
{
  return hb;
}

void unit :: setMoveSpeed (fixed ms)
{
  moveSpeed = ms;
}

void unit :: setUnitID (int id)
{
  unitID = id;
}

int unit :: getUnitID ()
{
  return unitID;
}

void unit :: setOwnerID (int oid)
{
  ownerID = oid;
}

int unit :: getOwnerID ()
{
  return ownerID;
}

void unit :: setUnitType (int id)
{
  unitType = id;
}

int unit :: getUnitType ()
{
  return unitType;
}

void unit :: setHP (int hp)
{
  hitPoints = hp;
}

int unit :: getHP ()
{
  return hitPoints;
}

void unit :: setMaxHP (int hp)
{
  maxHP = hp;
}

int unit :: getMaxHP ()
{
  return maxHP;
}

void unit :: setMoveType (int hp)
{
  moveType = hp;
}

int unit :: getMoveType ()
{
  return moveType;
}

void unit :: setImmuneToAilments (bool aa)
{
  immuneToAilments = aa;
}

void unit :: setTimeToLive (int hp)
{
  timeToLive = hp;
}

int unit :: getTimeToLive ()
{
  return timeToLive;
}

int unit :: getCurrentActivity ()
{
  return currentActivity;
}

int unit :: getInvicTime ()
{
  return statusAilment -> at (statusAilmentInvic);
}

int unit :: getStunTime ()
{
  return statusAilment -> at (statusAilmentStun);
}

int unit :: getNumHitsWhileStun ()
{
  return numHitsWhileStun;
}

void unit :: giveJuiceToPlayer (unit *playerUnit, int juice, int mult)
{
  //This gives 'juice', or hitPoints, to the playerUnit.
  //...unless the playerUnit is dead or has the 'no juice' status ailment.
  //...unless the playerUnit is 'suffering' from the 'more juice' status ailment. In which case he gets a triple juice bonus!
  if (playerUnit -> getStatusAilment (statusAilmentMoreJuice) > 0)
    mult *= 3;
  
  if (playerUnit -> getCurrentActivity () != charStatusDead && playerUnit -> getStatusAilment (statusAilmentNoJuice) <= 0 && playerUnit -> getHP () > 0)
    playerUnit -> pendingHPloss -= juice * mult;
}

int unit :: getStatusAilment (int name)
{
  return statusAilment -> at (name);
}

int unit :: getBiggestStatusTime (int name)
{
  return biggestStatusTime -> at (name);
}

int unit :: getActivity ()
{
  return currentActivity;
}

int unit :: getPendingActivity ()
{
  return pendingActivity;
}

void unit :: addStatusAilment (int name, int amount)
{
  statusAilment -> at (name) += amount;
}

void unit :: setDiesWhenHitsWall (bool gub)
{
  diesWhenHitsWall = gub;
}

bool unit :: getDiesWhenHitsWall ()
{
  return diesWhenHitsWall;
}

bool unit :: getHelpsPlayer ()
{
  return helpsPlayer;
}

void unit :: setIgnoreWall (bool iw)
{
  ignoreWall = iw;
}

bool unit :: getIgnoreWall ()
{
  return ignoreWall;
}

fixed unit :: getDrawScale ()
{
  return drawScale;
}

void unit :: setDrawScale (fixed gub)
{
  drawScale = gub;
}

fixed unit :: getKnockbackMul ()
{
  return knockbackMul;
}

void unit :: setKnockbackMul (fixed gub)
{
  knockbackMul = gub;
}

int unit :: getAffiliation ()
{
  return affiliation;
}

void unit :: setAffiliation (int hp)
{
  affiliation = hp;
}

unit :: ~unit ()
{

}
#endif
