#ifndef _UNIT_H_
#define _UNIT_H_

#include "allegro.h"
#include "unitCreate.h"

#include <vector>
using namespace std;

class unitList;
class unitGraphics;
class unitCreate;
class hitbox;
class dataObj;

/*
  The Unit class is one of most complicated classes I have. It handles
  1) Drawing
  2) Hit detection
  3) AI/player input

  battlemain calls each unit to process input (processInput), check for collisions and decrement status ailments (makePendingActions). Once all of the units have calculated what they are doing for this frame, they execute the results (processPendingActions).

  Specialized units (like playerUnit) inherit most of their methods and fields from unit. The virtual functions listed below may be overwritten in a more specific class.
 */

class unit {

public:

  //constructor
  unit ();

  //main functions
  virtual void init (int, int, int);
  virtual void draw (BITMAP *, bool);
  virtual void processInput (dataObj * dO, unitList *);
  virtual void makePendingActions (dataObj * dO, unitList *);
  virtual void processPendingActions (dataObj * dO, unitList *);
  virtual void addAttacks (dataObj * dO, unitList *, int);

  //secondary functions: checkers
  void collideCheck (dataObj * dO, unitList *);
  void statusCheck (dataObj * dO, unitList *);
  void arenaBoundaryCheck (fixed &, fixed &, unitGraphics *, dataObj *);

  //public accessors and modifiers
  void setDrawPos (int, int);
  int getDrawPosX ();
  int getDrawPosY ();

  void setPos (fixed, fixed);
  fixed getPosX ();
  fixed getPosY ();

  void setUnitGraphics (unitGraphics *);
  unitGraphics * getUnitGraphics ();
  void setHitbox (hitbox *);
  hitbox * getHitbox ();

  void setUnitID (int);
  int getUnitID ();
  void setOwnerID (int);
  int getOwnerID ();

  void setUnitType (int);
  int getUnitType ();

  int getCurrentActivity ();

  int getDeathTimeRemoval ();
  void setDeathTimeRemoval (int);

  int getInvicTime ();

  int getStunTime ();
  int getNumHitsWhileStun ();

  fixed getDrawAngle ();
  fixed getMoveAngle ();

  void setDrawAngle (fixed);
  void setMoveAngle (fixed);

  void setMoveSpeed (fixed);

  void setHP (int);
  int getHP ();
  void setMaxHP (int);
  int getMaxHP ();

  void giveJuiceToPlayer (unit *, int, int);

  void setMoveType (int);
  int getMoveType ();

  void setTimeToLive (int);
  int getTimeToLive ();

  void setDiesWhenHitsWall (bool);
  bool getDiesWhenHitsWall ();

  void setIgnoreWall (bool);
  bool getIgnoreWall ();

  int getStatusAilment (int);
  void addStatusAilment (int, int);

  int getBiggestStatusTime (int);

  int getActivity ();
  int getPendingActivity ();

  void setImmuneToAilments (bool);

  fixed getDrawScale ();
  void setDrawScale (fixed);

  fixed getKnockbackMul ();
  void setKnockbackMul (fixed);

  int getAffiliation ();
  void setAffiliation (int);

  bool getHelpsPlayer ();

  //destructor
  virtual ~unit ();

  int tempVar1;

protected:
  //unit identifiers
  int unitID;
  int unitType;
  int ownerID;
  int affiliation;

  //drawing and animation
  int drawX, drawY;
  fixed drawAngle;
  int picFrame;
  unitGraphics * uG;
  fixed drawScale;
  bool wantsToTaunt;

  //movement and location
  int moveType;
  fixed moveAngle;
  int moveMag;
  fixed moveSpeed;
  fixed posX, posY;
  fixed offsetX, offsetY;

  //hitbox and knockback
  hitbox * hb;
  fixed knockbackMul;
  fixed knockbackX;
  fixed knockbackY;
  int knockbackDrift;

  //vitals
  int hitPoints;
  int pendingHPloss;
  int maxHP;
  int hpToGivePlayer;
  int deathTimeRemoval;
  int timeToLive;
  bool diesWhenHitsWall;
  bool hitsWall;
  bool ignoreWall;
  bool immuneToAilments;
  bool diesWhenHitsAnything;
  bool hitSomething;
  bool helpsPlayer;

  //statuses
  int numHitsWhileStun;
  int cantAttackTime1;
  int cantAttackTime2;
  int cantAttackTime3;
  int cantMoveTime;
  int currentActivity;
  int pendingActivity;
  int pendingAttack;
  int currentAttack;
  vector <int> * statusAilment;
  vector <int> * biggestStatusTime;
};
#endif
