#ifndef _UNITLIST_CPP_
#define _UNITLIST_CPP_

#include "unitList.h"

#include "allegro.h"
#include "unit.h"
#include "unitCreate.h"
#include "unitGraphics.h"
#include "hitbox.h"
#include "gameConstants.h"
#include "loader.h"

#include <vector>
#include <iterator>
#include <list>
using namespace std;

unitList :: unitList ()
{
  units = new vector <unit *> (0);
  unitIDCount = 0;

  uCQ = new vector <unitCreate *> (0);
  uGQ = new vector <unitGraphics *> (0);
  hbQ = new vector <hitbox *> (0);

  lder = NULL;
}

void unitList :: clearAll ()
{
  units -> resize (0);
  unitIDCount = 0;
  uCQ -> resize (0);
}

void unitList :: addUnit (unit * un)
{
  //adds unit to the list, and sets its unitID to the next availible one.
  units -> push_back (un);
  un -> setUnitID (unitIDCount);
  unitIDCount++;
}

void unitList :: addGraphics (unitGraphics * uG)
{
  uGQ -> push_back (uG);
}

void unitList :: addHitbox (hitbox * hb)
{
  hbQ -> push_back (hb);
}

unit * unitList :: at (int unitID)
{
  return getUnit (unitID);
}

void unitList :: setLoader (loader * ld2)
{
  lder = ld2;
}

unit * unitList :: getUnit (int unid)
{
  unit * un = NULL;

  vector<unit *>::iterator the_iterator;

  if (unid == -1)
    return NULL;

  the_iterator = units -> begin();

  while(the_iterator < units -> end() && un == NULL) {
    if ((*the_iterator) -> getUnitID () == unid)
      un = (*the_iterator);
    the_iterator++;
  }
  
  return un;
}

vector <unit *> * unitList :: getUnits ()
{
  return units;
}

void unitList :: addCreate (unitCreate * uC)
{
  //adds unitcreate to the list of unit creates.
  uCQ -> push_back (uC);
}

void unitList :: createUnits ()
{
  //checks the list of create units.
  //It will decrement the timeToCreate of each until it is ready.
  //A new unit is created according to the unitCreate's parameters.

  unit * un = NULL;

  vector<unitCreate *>::iterator the_iterator;

  the_iterator = uCQ -> begin();

  while(the_iterator < uCQ -> end()) {

    un = NULL;

    if ((*the_iterator) -> timeToCreate <= 0)
      {
	if (lder != NULL)
	  lder -> loadUnit ((*the_iterator) -> unitType, un);

	if (un != NULL)
	  {
	    un -> setUnitType ((*the_iterator) -> unitType);
	    un -> setUnitGraphics (uGQ -> at (searchGraphics ((*the_iterator) -> unitType)));

	    un -> setMoveAngle ((*the_iterator) -> faceAngle);
	    un -> setPos ((*the_iterator) -> posX, (*the_iterator) -> posY);
	    un -> setHitbox (hbQ -> at (searchHitBox ((*the_iterator) -> unitType)));
	    un -> setOwnerID ((*the_iterator) -> ownerID);
	    un -> init ((*the_iterator) -> param1, (*the_iterator) -> param2, (*the_iterator) -> param3);
	    addUnit (un);
	    uCQ -> erase (the_iterator);
	  }
      }
    else
      (*the_iterator) -> timeToCreate--;

    the_iterator++;
  }
}

void unitList :: cullUnits ()
{
  //removes dead units.
  vector<unit *>::iterator the_iterator;

  the_iterator = units -> begin();

  while(the_iterator < units -> end()) {
    if ((*the_iterator) -> getDeathTimeRemoval () <= 0 && ((*the_iterator)) -> getHP () <= 0)
	{
	  units -> erase (the_iterator);
	}
      the_iterator++;
  }
}

int unitList ::searchHitBox (int spec1)
{
  int ans = 0, i = 0;

  for (i = 0; i < int (hbQ -> size ()); i ++)
    {
      if (hbQ -> at (i) -> unitType == spec1)
	ans = i;
    }
  return ans;
}

int unitList ::searchGraphics (int spec1)
{
  int ans = 0, i = 0;
  
  for (i = 0; i < int (uGQ -> size ()); i ++)
    {
      if (uGQ -> at (i) -> unitType == spec1)
	ans = i;
    }
  return ans;
}

vector <unit *> * unitList :: search (vector <unit *> * vec, int criterion, int spec1)
{
  vector <unit *> * ans;
  int temp;
  unit * un = NULL;
  ans = new vector <unit *> (0);

  //look through units, matching criterion and spec
  vector<unit *>::iterator the_iterator;

  the_iterator = vec -> begin();

  while(the_iterator < vec -> end()) {
      switch (criterion)
	{
	case (searchByUnitType):
	  if ((*the_iterator) -> getUnitType () == spec1)
	    ans -> push_back ((*the_iterator));
	  break;
	case (searchByOwner):
	  temp = (*the_iterator) -> getOwnerID ();
	  un = getUnit (temp);
	  if (un != NULL && un -> getUnitType () == spec1)
	    ans -> push_back ((*the_iterator));
	  break;
	case (searchByAffiliation):
	  if ((*the_iterator) -> getAffiliation () == spec1)
	    ans -> push_back ((*the_iterator));
	  break;
	default: break;
	}
      the_iterator++;
    }
  return ans;
}

bool unitList :: anyRemainingAffiliation (int affil)
{
  //look through units and unitCreates, searching for spec
  vector<unit *>::iterator iter1;
  vector<unitCreate *>::iterator iter2;

  iter1 = units -> begin();
  iter2 = uCQ -> begin();

  while(iter1 < units -> end()) {
    if ((*iter1) -> getAffiliation () == affil)
      return true;
    iter1++;
  }

  while(iter2 < uCQ -> end()) {
    if (lder -> getAffiliationByUnit ((*iter2) -> unitType) == affil)
      return true;
    iter2++;
  }

  return false;
}

int unitList :: latestTime ()
{
  //look through the unit creates, and find the latest time that a unit is to be created.

  vector<unitCreate *>::iterator iter;
  int ans = 0;

  iter = uCQ -> begin();

  while(iter < uCQ -> end()) {
    if ((*iter) -> timeToCreate > ans)
      ans = (*iter) -> timeToCreate;
    iter++;
  }

  return ans;
}

unitList :: ~unitList ()
{

}
#endif
