#ifndef _VIDEOSETTINGS_CPP_
#define _VIDEOSETTINGS_CPP_

#include <stdio.h>
#include <math.h>

#include "allegro.h"

#include "videoSettings.h"

#include <vector>
using namespace std;

videoSettings :: videoSettings ()
{
  cameraX = 160;
  cameraY = 50;

  resolutionX = 640;
  resolutionY = 480;

  backgroundR = 48;
  backgroundG = 48;
  backgroundB = 48;

  arenaPosX = 0;
  arenaPosY = 0;
  arenaScale = ftofix (1.0);

  hpStartX = 0;
  hpStartY = 0;
  hpSizeX = 0; 
  hpSizeY = 0;

  colorDepth = 24;

  windowed = false;
}

void videoSettings :: setObjectPositions (int arenSiX, int arenSiY)
{
  //this sets up position and size variables for the main screen, which battlemain uses.

  //first, get the scaling down, based on the arena's dimensions.
  //they should be scaled so neither one exceeds 70% of the width or the height of the resolution.

  if (arenSiX <= 0 || arenSiY <= 0)
    {
      //handling error case
      //huh? An arena must have positive dimensions.
      arenaScale = ftofix (1.0);
      arenaPosX = 0;
      arenaPosY = resolutionY;
    }
  else if (arenSiX >= ((arenSiY * resolutionX) / resolutionY))
    {
      arenaScale = ftofix ((0.70 * resolutionX) / float (arenSiX));
      arenaPosX = (resolutionX - fixtoi(arenSiX * arenaScale)) / 2;
      arenaPosY = (resolutionY - fixtoi(arenSiY * arenaScale)) / 2;
    }
  else
    {
      arenaScale = ftofix ((0.70 * resolutionY) / float (arenSiY));
      arenaPosX = (resolutionX - fixtoi(arenSiX * arenaScale)) / 2;
      arenaPosY = (resolutionY - fixtoi(arenSiY * arenaScale)) / 2;
    }

  
  //size of hp bar = 70% of width x 10% of height. It is centered along the y axis, right above the arena.
  hpStartX = (int) (resolutionX * .15);
  hpStartY = arenaPosY - 10 - (int) (resolutionY * .05);

  hpSizeX = (int) (resolutionX * .7);
  hpSizeY = (int) (resolutionY * .05);

  return;
}

//  int cameraX, cameraY;
void videoSettings :: getCamera (int & camX, int & camY)
{
  camX = cameraX;
  camY = cameraY;
}

void videoSettings :: setCamera (int camX, int camY)
{
  cameraX = camX;
  cameraY = camY;
}

//  int resolutionX, resolutionY;
void videoSettings :: getResolution (int & resX, int & resY)
{
  resX = resolutionX;
  resY = resolutionY;
}

int videoSettings :: getResolutionX ()
{
  return resolutionX;
}

int videoSettings :: getResolutionY ()
{
  return resolutionY;
}

void videoSettings :: setResolution (int resX, int resY)
{
  resolutionX = resX;
  resolutionY = resY;
}

//  int backgroundR, backgroundG, backgroundB;
void videoSettings :: getBackgroundColor (int & bkcR, int & bkcG, int & bkcB)
{
  bkcR = backgroundR;
  bkcG = backgroundG;
  bkcB = backgroundB;
}

void videoSettings :: setBackgroundColor (int bkcR, int bkcG, int bkcB)
{
  backgroundR = bkcR;
  backgroundG = bkcG;
  backgroundB = bkcB;
}

int videoSettings :: getArenaPosX ()
{return arenaPosX;}

int videoSettings :: getArenaPosY ()
{return arenaPosY;}

int videoSettings :: getHPBarStartX ()
{return hpStartX;}

int videoSettings :: getHPBarStartY ()
{return hpStartY;}

int videoSettings :: getHPBarSizeX ()
{return hpSizeX;}

int videoSettings :: getHPBarSizeY ()
{return hpSizeY;}

fixed videoSettings :: getArenaScale ()
{return arenaScale;}

bool videoSettings :: getWindowed ()
{
  return windowed;
}

void videoSettings :: setWindowed (bool wind)
{
  windowed = wind;
}

int videoSettings :: getColorDepth ()
{
  return colorDepth;
}

void videoSettings :: setColorDepth (int cd)
{
  colorDepth = cd;
}

videoSettings ::  ~videoSettings ()
{

}
#endif
